@@ -1,11 +1,14 @@
 #include <R.h>
 #include <Rinternals.h>
+#include <string.h>
 
 #pragma push_macro("ERROR")
 #undef ERROR // mute GCC warning: "ERROR" redefined
 #include "model.h"
 #include "input.h"
 #include "params.h"
+#include "constant.h"
+#include "interventions.h"
 #pragma pop_macro("ERROR")
 
 SEXP R_get_app_users ( SEXP R_c_model, SEXP n_total )
@@ -95,15 +98,15 @@ SEXP R_get_individuals ( SEXP R_c_model, SEXP n_total )
   return R_list;
 }
 
-SEXP R_get_network_ids ( SEXP R_c_model, SEXP R_max_ids )
+SEXP R_get_network_ids ( SEXP R_c_model )
 {
   // get the point to the model from the R pointer object
   model *c_model = (model *) R_ExternalPtrAddr(R_c_model);
-  int max_ids = asInteger( R_max_ids );
+  int max_ids = c_model->n_networks;
 
   // allocate memory to for the function call
   int *ids = calloc( max_ids, sizeof(int) );
-  int n_ids = get_network_ids(c_model,ids,max_ids);
+  int n_ids = get_network_ids(c_model,ids);
 
   if( n_ids == -1 )
     return( ScalarInteger(-1));
@@ -138,8 +141,8 @@ SEXP R_get_transmissions ( SEXP R_c_model )
   int n_trans = get_n_transmissions(c_model);
 
   // allocate memory to for the function call
-  const int n_names = 33;
-  const char *names[33] = { "ID_recipient", "age_group_recipient",
+  const int n_names = 35;
+  const char *names[35] = { "ID_recipient", "age_group_recipient",
     "house_no_recipient","occupation_network_recipient","worker_type_recipient",
     "hospital_state_recipient","infector_network","infector_network_id",
     "generation_time","ID_source","age_group_source",
@@ -149,8 +152,8 @@ SEXP R_get_transmissions ( SEXP R_c_model )
     "time_presymptomatic_severe","time_symptomatic","time_symptomatic_mild",
     "time_symptomatic_severe","time_asymptomatic","time_hospitalised",
     "time_critical","time_hospitalised_recovering","time_death",
-    "time_recovered","time_susceptible","is_case",
-    "strain_multiplier" };
+    "time_recovered","time_susceptible","is_case", "strain_idx",
+    "transmission_multiplier", "expected_hospitalisation" };
   long *ID_recipient = calloc( n_trans, sizeof(long) );
   int *age_group_recipient = calloc( n_trans, sizeof(int) );
   long *house_no_recipient = calloc( n_trans, sizeof(long) );
@@ -183,7 +186,9 @@ SEXP R_get_transmissions ( SEXP R_c_model )
   int *time_recovered = calloc( n_trans, sizeof(int) );
   int *time_susceptible = calloc( n_trans, sizeof(int) );
   int *is_case = calloc( n_trans, sizeof(int) );
-  float *strain_multiplier = calloc( n_trans, sizeof(float) );
+  int *strain_idx = calloc( n_trans, sizeof(int) );
+  float *transmission_multiplier = calloc( n_trans, sizeof(float) );
+  float *expected_hospitalisation = calloc( n_trans, sizeof(float) );
 
   get_transmissions( c_model, ID_recipient, age_group_recipient,
       house_no_recipient, occupation_network_recipient, worker_type_recipient,
@@ -194,7 +199,8 @@ SEXP R_get_transmissions ( SEXP R_c_model )
       time_presymptomatic_mild, time_presymptomatic_severe, time_symptomatic,
       time_symptomatic_mild, time_symptomatic_severe, time_asymptomatic,
       time_hospitalised, time_critical, time_hospitalised_recovering,
-      time_death, time_recovered, time_susceptible, is_case,strain_multiplier );
+      time_death, time_recovered, time_susceptible, is_case, strain_idx,
+      transmission_multiplier, expected_hospitalisation );
 
   // convert to R object
   SEXP R_list = PROTECT(allocVector(VECSXP, n_names));
@@ -231,7 +237,9 @@ SEXP R_get_transmissions ( SEXP R_c_model )
   SEXP R_time_recovered = PROTECT(allocVector(INTSXP, n_trans));
   SEXP R_time_susceptible = PROTECT(allocVector(INTSXP, n_trans));
   SEXP R_is_case = PROTECT(allocVector(INTSXP, n_trans));
-  SEXP R_strain_multiplier = PROTECT(allocVector(REALSXP, n_trans));
+  SEXP R_strain_idx = PROTECT(allocVector(INTSXP, n_trans));
+  SEXP R_transmission_multiplier = PROTECT(allocVector(REALSXP, n_trans));
+  SEXP R_expected_hospitalisation = PROTECT(allocVector(REALSXP, n_trans));
 
   for( int i = 0; i < n_trans; i++ ) {
     INTEGER(R_ID_recipient)[i] = ID_recipient[i];
@@ -266,7 +274,9 @@ SEXP R_get_transmissions ( SEXP R_c_model )
     INTEGER(R_time_recovered)[i] = time_recovered[i];
     INTEGER(R_time_susceptible)[i] = time_susceptible[i];
     INTEGER(R_is_case)[i] = is_case[i];
-    REAL(R_strain_multiplier)[i] = strain_multiplier[i];
+    INTEGER(R_strain_idx)[i] = strain_idx[i];
+    REAL(R_transmission_multiplier)[i] = transmission_multiplier[i];
+    REAL(R_expected_hospitalisation)[i] = expected_hospitalisation[i];
   }
 
   SET_VECTOR_ELT(R_list, 0, R_ID_recipient);
@@ -301,7 +311,9 @@ SEXP R_get_transmissions ( SEXP R_c_model )
   SET_VECTOR_ELT(R_list, 29, R_time_recovered);
   SET_VECTOR_ELT(R_list, 30, R_time_susceptible);
   SET_VECTOR_ELT(R_list, 31, R_is_case);
-  SET_VECTOR_ELT(R_list, 32, R_strain_multiplier);
+  SET_VECTOR_ELT(R_list, 32, R_strain_idx);
+  SET_VECTOR_ELT(R_list, 33, R_transmission_multiplier);
+  SET_VECTOR_ELT(R_list, 34, R_expected_hospitalisation);
 
   for (int i = 0; i < n_names; i++)
     SET_STRING_ELT(R_Names, i, mkChar(names[i]));
@@ -340,12 +352,192 @@ SEXP R_get_transmissions ( SEXP R_c_model )
   free( time_recovered );
   free( time_susceptible );
   free( is_case );
-  free( strain_multiplier );
+  free( strain_idx );
+  free( transmission_multiplier );
+  free( expected_hospitalisation );
   UNPROTECT(n_names+2);
 
   return R_list;
 }
 
+SEXP R_add_new_strain ( SEXP R_c_model, SEXP R_transisition_multiplier,
+                        SEXP R_hospitalised_fraction )
+{
+  // get the point to the model from the R pointer object
+  model *c_model = (model *) R_ExternalPtrAddr(R_c_model);
 
+  // allocate memory to for the function call
+  float transition_multiplier = asReal( R_transisition_multiplier );
+  double *hospitalised_fraction = calloc( N_AGE_GROUPS, sizeof(double) );
 
+  for( int i = 0; i < N_AGE_GROUPS; i++ )
+    hospitalised_fraction[ i ] = REAL(R_hospitalised_fraction )[ i ];
 
+  int n_strain = add_new_strain(c_model,transition_multiplier, hospitalised_fraction);
+
+  free( hospitalised_fraction );
+
+  return ScalarInteger( n_strain );
+}
+
+SEXP R_add_vaccine ( SEXP R_c_model, SEXP R_full_efficacy, SEXP R_symptoms_efficacy,
+    SEXP R_severe_efficacy, SEXP R_time_to_protect, SEXP R_vaccine_protection_period )
+{
+  // get the point to the model from the R pointer object
+  model *c_model = (model *) R_ExternalPtrAddr(R_c_model);
+
+  // allocate memory to for the function call
+  short n_strains = c_model->params->max_n_strains;
+  float *full_efficacy = calloc( n_strains, sizeof(float) );
+  float *symptoms_efficacy = calloc( n_strains, sizeof(float) );
+  float *severe_efficacy = calloc( n_strains, sizeof(float) );
+  short time_to_protect = asInteger( R_time_to_protect );
+  short vaccine_protection_period = asInteger( R_vaccine_protection_period );
+
+  for( int i = 0; i < n_strains; i++ )
+  {
+    full_efficacy [ i ] = REAL(R_full_efficacy )[ i ];
+    symptoms_efficacy[ i ] = REAL(R_symptoms_efficacy )[ i ];
+    severe_efficacy[ i ] = REAL(R_severe_efficacy )[ i ];
+  }
+
+  short idx = add_vaccine( c_model,full_efficacy, symptoms_efficacy, severe_efficacy,
+                           time_to_protect,vaccine_protection_period );
+
+  free( full_efficacy  );
+  free( symptoms_efficacy );
+  free( severe_efficacy );
+
+  return ScalarInteger( idx);
+}
+
+SEXP R_vaccine_full_efficacy ( SEXP R_c_vaccine )
+{
+  // get the point to the model from the R pointer object
+  vaccine *c_vaccine = (vaccine *) R_ExternalPtrAddr(R_c_vaccine);
+  int n_strains = c_vaccine->n_strains;
+
+  // convert to R object
+  SEXP R_res = PROTECT(allocVector(REALSXP, n_strains));
+  for( int i = 0; i < n_strains; i++ )
+    REAL(R_res)[i] = c_vaccine->full_efficacy[i];
+
+  // free the memory
+  UNPROTECT(1);
+
+  return R_res;
+}
+
+SEXP R_vaccine_severe_efficacy ( SEXP R_c_vaccine )
+{
+  // get the point to the model from the R pointer object
+  vaccine *c_vaccine = (vaccine *) R_ExternalPtrAddr(R_c_vaccine);
+  int n_strains = c_vaccine->n_strains;
+
+  // convert to R object
+  SEXP R_res = PROTECT(allocVector(REALSXP, n_strains));
+  for( int i = 0; i < n_strains; i++ )
+    REAL(R_res)[i] = c_vaccine->severe_efficacy[i];
+
+  // free the memory
+  UNPROTECT(1);
+
+  return R_res;
+}
+
+SEXP R_vaccine_symptoms_efficacy ( SEXP R_c_vaccine )
+{
+  // get the point to the model from the R pointer object
+  vaccine *c_vaccine = (vaccine *) R_ExternalPtrAddr(R_c_vaccine);
+  int n_strains = c_vaccine->n_strains;
+
+  // convert to R object
+  SEXP R_res = PROTECT(allocVector(REALSXP, n_strains));
+  for( int i = 0; i < n_strains; i++ )
+    REAL(R_res)[i] = c_vaccine->symptoms_efficacy[i];
+
+  // free the memory
+  UNPROTECT(1);
+
+  return R_res;
+}
+
+SEXP R_intervention_vaccinate_age_group ( SEXP R_c_model,
+      SEXP R_fraction_to_vaccinate, SEXP R_c_vaccine )
+{
+  // get the point to the model from the R pointer object
+  vaccine *c_vaccine = (vaccine *) R_ExternalPtrAddr(R_c_vaccine);
+  model *c_model = (model *) R_ExternalPtrAddr(R_c_model);
+
+  long *total_vaccinated = calloc( N_AGE_GROUPS, sizeof(long) );
+  double *fraction_to_vaccinate= calloc( N_AGE_GROUPS, sizeof(double) );
+
+  for( int i = 0; i < N_AGE_GROUPS; i++ )
+  {
+    total_vaccinated[ i ]      = 0;
+    fraction_to_vaccinate[ i ] = REAL(R_fraction_to_vaccinate )[ i ];
+  }
+
+  long n_vac =intervention_vaccinate_age_group( c_model, fraction_to_vaccinate,
+                                                c_vaccine, total_vaccinated );
+
+  // convert to R object
+  SEXP R_res = PROTECT(allocVector(INTSXP, N_AGE_GROUPS));
+  for( int i = 0; i < N_AGE_GROUPS; i++ )
+    INTEGER(R_res)[i] = total_vaccinated[i];
+
+  // free the memory
+  free( total_vaccinated );
+  free( fraction_to_vaccinate );
+  UNPROTECT(1);
+
+  return R_res;
+}
+
+SEXP R_set_input_param_file( SEXP R_c_params,
+                         SEXP R_input_param_file )
+{
+  parameters *c_params = (parameters *) R_ExternalPtrAddr(R_c_params);
+
+  const char* input_param_file = CHAR( asChar( R_input_param_file ) );
+
+  strncpy( c_params->input_param_file, input_param_file, sizeof( c_params->input_param_file ) - 1 );
+
+  return ScalarInteger( 1 );
+}
+
+SEXP R_set_input_household_file( SEXP R_c_params,
+                             SEXP R_input_household_file )
+{
+  parameters *c_params = (parameters *) R_ExternalPtrAddr(R_c_params);
+
+  const char* input_household_file = CHAR( asChar( R_input_household_file ) );
+
+  strncpy( c_params->input_household_file, input_household_file, sizeof( c_params->input_household_file ) - 1 );
+
+  return ScalarInteger( 1 );
+}
+
+SEXP R_set_output_file_dir( SEXP R_c_params,
+                            SEXP R_output_file_dir )
+{
+  parameters *c_params = (parameters *) R_ExternalPtrAddr(R_c_params);
+
+  const char* output_file_dir = CHAR( asChar( R_output_file_dir ) );
+
+  strncpy( c_params->output_file_dir, output_file_dir, sizeof( c_params->output_file_dir ) - 1 );
+
+  return ScalarInteger( 1 );
+}
+
+SEXP R_set_hospital_input_param_file( SEXP R_c_params,
+                             SEXP R_input_param_file )
+{
+  parameters *c_params = (parameters *) R_ExternalPtrAddr(R_c_params);
+
+  const char* input_param_file = CHAR( asChar( R_input_param_file ) );
+
+  strncpy( c_params->hospital_input_param_file, input_param_file, sizeof( c_params->hospital_input_param_file ) - 1 );
+
+  return ScalarInteger( 1 );
+}
